Ralpb = LibStub("AceAddon-3.0"):NewAddon("RalpbQuest", "AceConsole-3.0", "AceEvent-3.0")

-- Local vars
local CURRENT_VERSION = GetAddOnMetadata("RalpbQuest", "Version")
local RQ_CHAT_TYPE = "PARTY"
local RQ_QUEST_NOTIFY = true

-- Start up 
function Ralpb:OnEnable()
    self:RegisterSlashCommands()

    self.db.char.taskStorage = {}

    self:RegisterEvent("PARTY_MEMBERS_CHANGED")
    self:RegisterEvent("UNIT_QUEST_LOG_CHANGED")

    if self:InParty() then
        RQ_CHAT_TYPE = "PARTY"
    else
        RQ_CHAT_TYPE = "SAY"
    end

    --self:InitQuestArray()
end

function Ralpb:OnInitialize()
    self.db = LibStub("AceDB-3.0"):New("RalpbQuestDB")

    if self.db.char.notify == nil then
        RQ_QUEST_NOTIFY = true
        self.db.char.notify = RQ_QUEST_NOTIFY
    else
        RQ_QUEST_NOTIFY = self.db.char.notify
    end
end

function Ralpb:Announce()
    local numEntries, numQuests = GetNumQuestLogEntries()
    local questCompleted = true

    for questIndex = 1, numEntries do
        SelectQuestLogEntry(questIndex)
        local title, level, suggestedGroup, isHeader, isCollapsed, isComplete, frequency, questID, startEvent, displayQuestID, isOnMap, hasLocalPOI, isTask, isStory, isHidden = GetQuestLogTitle(questIndex)
        local numberOfTasks = GetNumQuestLeaderBoards(questIndex)

        if not isHeader then

            if (not self.db.char.taskStorage[questID]) then
                self.db.char.taskStorage[questID] = { questID = questID, isComplete = isComplete }
            end

            if isComplete == nil or isComplete == -1 then
                questCompleted = false
            end

            if not questCompleted then
                -- Opslaan objectives per questID
                for boardIndex = 1, numberOfTasks do

                    local objDesc, objType, objComplete = GetQuestLogLeaderBoard(boardIndex, questIndex)

                    local storeObjCurrent, storeObjTotal, storeObjText

                    if (objType == "progressbar") then
                        storeObjCurrent = percent
                        storeObjTotal = 100
                        storeObjText = objDesc
                    else
                        storeObjCurrent, storeObjTotal, storeObjText = string.match(objDesc, "(%d+)/(%d+) ?(.*)")
                        storeObjCurrent = tonumber(storeObjCurrent)
                        storeObjTotal = tonumber(storeObjTotal)
                    end

                    if self.db.char.taskStorage[questID] then
                        if (self.db.char.taskStorage[questID].isComplete == nil or self.db.char.taskStorage[questID].isComplete == -1) then
                            if (not self.db.char.taskStorage[questID][boardIndex]) then
                                self.db.char.taskStorage[questID][boardIndex] = {taskObjCurrent = storeObjCurrent, taskObjTotal = storeObjTotal, taskObjText = storeObjText}

                                if storeObjCurrent ~= 0 then
                                    self:SendNotification(storeObjCurrent .. "/" .. storeObjTotal .. " " .. storeObjText)
                                end
                            elseif (self.db.char.taskStorage[questID][boardIndex].taskObjCurrent ~= storeObjCurrent) then
                                self.db.char.taskStorage[questID][boardIndex].taskObjCurrent = storeObjCurrent
                                self:SendNotification(storeObjCurrent .. "/" .. storeObjTotal .. " " .. storeObjText)
                            end
                        end
                    end                
                end
            else
                -- Quest is completed
                if numberOfTasks > 0 and (self.db.char.taskStorage[questID].isComplete == nil or self.db.char.taskStorage[questID].isComplete == -1) then
                    local link = GetQuestLink(questID)

                    -- Quest is completed
                    PlaySoundFile("sound/interface/MapPing.wav")
                    self:SendNotification(link .. " " .. "is now completed!")
                end

                self.db.char.taskStorage[questID].isComplete = questCompleted
            end
        end

        questCompleted = true
        
    end
end

-- Events
function Ralpb:UNIT_QUEST_LOG_CHANGED()

    self:Announce()

end

function Ralpb:PARTY_MEMBERS_CHANGED()
    if self:InParty() then
        RQ_CHAT_TYPE = "PARTY"
    else
        RQ_CHAT_TYPE = "SAY"
    end
end

function Ralpb:InParty()
    local returnValue = false

    if GetNumGroupMembers() > 0 then
            returnValue = true
    end

    return returnValue
end

-- Slash commands
function Ralpb:RegisterSlashCommands()
    self:RegisterChatCommand('ralpb', 'OnCmd')
end

function Ralpb:OnCmd(cmd)
    if cmd == 'reloadui' then
        self:Reload()
    elseif cmd == 'version' then
        self:PrintVersion()
    elseif cmd == 'party' then
        if self:InParty() then
            RQ_CHAT_TYPE = "PARTY"
            self:Print("Notify quest progression into the party chat.")
        end
    elseif cmd == 'say' then
        RQ_CHAT_TYPE = "SAY"
        self:Print("Notify quest progression into the say chat.")
    elseif cmd == 'enable' then
        Ralpb:Enable()
    else
        self:PrintInfo()
    end
end

function Ralpb:Enable()
    if RQ_QUEST_NOTIFY then
        RQ_QUEST_NOTIFY = false
        self:Print("Notify quest progression in chat is disabled.")
    else
        RQ_QUEST_NOTIFY = true
        self:Print("Notify quest progression in chat is enabled.")
    end

    self.db.char.notify = RQ_QUEST_NOTIFY
end

function Ralpb:PrintVersion()
    self:Print("Version: " .. CURRENT_VERSION)
end

function Ralpb:Reload()
    ReloadUI()
end

function Ralpb:PrintInfo()
    self:Print("RalpbQuest is an addon made by Ralpb on Bloodscalp")
end

function Ralpb:SendNotification(message)
    if RQ_QUEST_NOTIFY then
        SendChatMessage(message, RQ_CHAT_TYPE);
    end
end

-- Help functions
local function IsEmpty(s)
    return s == nil or s == ''
end